# Ship.py
# March 16, 2002
#
# by Evan Light aka sleight42, all rights reserved
#
# Permission to redistribute this code as part of any other packaging requires
# the explicit permission of the author in advance.
##############################################################################

import App
import MissionLib
import ftb.Ship
import ftb.ShipManager

# Define our events

# Fire primary weapons on secondary targets
ET_INPUT_FIRE_PRIMARY_ON_LIST = App.Game_GetNextEventType()

# Fire secondary weapons on secondary targets
ET_INPUT_FIRE_SECONDARY_ON_LIST = App.Game_GetNextEventType()

# Fire tertiary weapons on secondary targets
ET_INPUT_FIRE_TERTIARY_ON_LIST = App.Game_GetNextEventType()

# Clear the secondary targets list
ET_INPUT_CLEAR_SECONDARY_TARGETS= App.Game_GetNextEventType()

# Toggle a secondary target in or out of the list
ET_INPUT_TOGGLE_SECONDARY_TARGET= App.Game_GetNextEventType()

def MissionStart( pObject, pEvent):
    RegisterEventHandlers()
    ClearOldKeyBindings()
    MapNewKeyBindings()

#### BEGIN EVENT HANDLERS
def FirePrimaryWeaponsOnList( pObject, pEvent):
    pShip = App.Game_GetCurrentPlayer()
    pMyShip = ftb.ShipManager.GetShip( pShip)
    pMyShip.FireWeaponsOnList( pEvent.GetBool(), App.ShipClass.WG_PRIMARY)
    pObject.CallNextHandler(pEvent)

def FireSecondaryWeaponsOnList( pObject, pEvent):
    pShip = App.Game_GetCurrentPlayer()
    pMyShip = ftb.ShipManager.GetShip( pShip)
    pMyShip.FireWeaponsOnList( pEvent.GetBool(), App.ShipClass.WG_SECONDARY)
    pObject.CallNextHandler(pEvent)

def FireTertiaryWeaponsOnList( pObject, pEvent):
    pShip = App.Game_GetCurrentPlayer()
    pMyShip = ftb.ShipManager.GetShip( pShip)
    pMyShip.FireWeaponsOnList( pEvent.GetBool(), App.ShipClass.WG_TERTIARY)
    pObject.CallNextHandler(pEvent)

def ClearSecondaryTargets(pObject, pEvent):
    pPlayer = App.Game_GetCurrentPlayer()
    if pPlayer:
        pMyShip = ftb.ShipManager.GetShip( pPlayer)
        pMyShip.ClearSecondaryTargets() 

def ToggleSecondaryTarget(pObject, pEvent):
    pPlayer = App.Game_GetCurrentPlayer()
    if pPlayer:
        pMyShip = ftb.ShipManager.GetShip( pPlayer)
        pMyShip.ToggleSecondaryTarget( GetTargetByIdx( pEvent.GetInt())) 
#### END EVENT HANDLERS

def GetPlayerSet():
    pGame = App.Game_GetCurrentGame()
    return pGame.GetPlayerSet()

def GetTargetByIdx( idx):
    retval = None
    pEnemyGroup = MissionLib.GetEnemyGroup()
    lEnemies = pEnemyGroup.GetActiveObjectTupleInSet( GetPlayerSet())
    eCounter = 0
    for pEnemy in lEnemies:
        if( eCounter == idx ):
            retval = pEnemy
            break
        eCounter = eCounter + 1
    return retval

def RegisterEventHandlers():
    #### REGISTER EVENT HANDLERS
    lEventHandlerMap = (
        (ET_INPUT_CLEAR_SECONDARY_TARGETS, __name__ + ".ClearSecondaryTargets"),
        (ET_INPUT_TOGGLE_SECONDARY_TARGET, __name__ + ".ToggleSecondaryTarget"),
        (ET_INPUT_FIRE_TERTIARY_ON_LIST, __name__ + ".FireTertiaryWeaponsOnList"),
        (ET_INPUT_FIRE_SECONDARY_ON_LIST, __name__ + ".FireSecondaryWeaponsOnList"),
        (ET_INPUT_FIRE_PRIMARY_ON_LIST, __name__ + ".FirePrimaryWeaponsOnList"),
        )

    for eType, sFunc in lEventHandlerMap:
        App.g_kEventManager.AddBroadcastPythonFuncHandler( eType, App.Game_GetCurrentGame(), sFunc)
            
def ClearOldKeyBindings():
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_SELECT_X, App.KeyboardBinding.GET_INT_EVENT, 1)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_SELECT_X, App.KeyboardBinding.GET_INT_EVENT, 2)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_SELECT_X, App.KeyboardBinding.GET_INT_EVENT, 3)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_SELECT_X, App.KeyboardBinding.GET_INT_EVENT, 4)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_SELECT_X, App.KeyboardBinding.GET_INT_EVENT, 5)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_SELECT_X, App.KeyboardBinding.GET_INT_EVENT, 6)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_SELECT_X, App.KeyboardBinding.GET_INT_EVENT, 7)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_SELECT_X, App.KeyboardBinding.GET_INT_EVENT, 8)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_SELECT_X, App.KeyboardBinding.GET_INT_EVENT, 9)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_FIRE_PRIMARY, App.KeyboardBinding.GET_BOOL_EVENT, 0)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_FIRE_SECONDARY, App.KeyboardBinding.GET_BOOL_EVENT, 0)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_FIRE_TERTIARY, App.KeyboardBinding.GET_BOOL_EVENT, 0)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_FIRE_PRIMARY, App.KeyboardBinding.GET_BOOL_EVENT, 1)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_FIRE_SECONDARY, App.KeyboardBinding.GET_BOOL_EVENT, 1)
    App.g_kKeyboardBinding.ClearBinding(App.ET_INPUT_FIRE_TERTIARY, App.KeyboardBinding.GET_BOOL_EVENT, 1)


def MapNewKeyBindings():
    App.g_kKeyboardBinding.BindKey(App.WC_F, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_FIRE_PRIMARY_ON_LIST, App.KeyboardBinding.GET_BOOL_EVENT, 1)
    App.g_kKeyboardBinding.BindKey(App.WC_LBUTTON, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_FIRE_PRIMARY_ON_LIST, App.KeyboardBinding.GET_BOOL_EVENT, 1)
    App.g_kKeyboardBinding.BindKey(App.WC_F, App.TGKeyboardEvent.KS_KEYUP, ET_INPUT_FIRE_PRIMARY_ON_LIST, App.KeyboardBinding.GET_BOOL_EVENT, 0)
    App.g_kKeyboardBinding.BindKey(App.WC_LBUTTON, App.TGKeyboardEvent.KS_KEYUP, ET_INPUT_FIRE_PRIMARY_ON_LIST, App.KeyboardBinding.GET_BOOL_EVENT, 0)
    App.g_kKeyboardBinding.BindKey(App.WC_X, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_FIRE_SECONDARY_ON_LIST, App.KeyboardBinding.GET_BOOL_EVENT, 1)
    App.g_kKeyboardBinding.BindKey(App.WC_X, App.TGKeyboardEvent.KS_KEYUP, ET_INPUT_FIRE_SECONDARY_ON_LIST, App.KeyboardBinding.GET_BOOL_EVENT, 0)
    App.g_kKeyboardBinding.BindKey(App.WC_RBUTTON, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_FIRE_SECONDARY_ON_LIST, App.KeyboardBinding.GET_BOOL_EVENT, 1)
    App.g_kKeyboardBinding.BindKey(App.WC_RBUTTON, App.TGKeyboardEvent.KS_KEYUP, ET_INPUT_FIRE_SECONDARY_ON_LIST, App.KeyboardBinding.GET_BOOL_EVENT, 0)
    App.g_kKeyboardBinding.BindKey(App.WC_G, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_FIRE_TERTIARY_ON_LIST, App.KeyboardBinding.GET_BOOL_EVENT, 1)
    App.g_kKeyboardBinding.BindKey(App.WC_G, App.TGKeyboardEvent.KS_KEYUP, ET_INPUT_FIRE_TERTIARY_ON_LIST, App.KeyboardBinding.GET_BOOL_EVENT, 0)
    App.g_kKeyboardBinding.BindKey(App.WC_NUMPADENTER, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_CLEAR_SECONDARY_TARGETS, App.KeyboardBinding.GET_INT_EVENT, 0, App.KeyboardBinding.KBT_SINGLE_KEY_TO_EVENT)
    App.g_kKeyboardBinding.BindKey(App.WC_NUMPAD2, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_TOGGLE_SECONDARY_TARGET, App.KeyboardBinding.GET_INT_EVENT, 1, App.KeyboardBinding.KBT_SINGLE_KEY_TO_EVENT)
    App.g_kKeyboardBinding.BindKey(App.WC_NUMPAD3, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_TOGGLE_SECONDARY_TARGET, App.KeyboardBinding.GET_INT_EVENT, 2, App.KeyboardBinding.KBT_SINGLE_KEY_TO_EVENT)
    App.g_kKeyboardBinding.BindKey(App.WC_NUMPAD4, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_TOGGLE_SECONDARY_TARGET, App.KeyboardBinding.GET_INT_EVENT, 3, App.KeyboardBinding.KBT_SINGLE_KEY_TO_EVENT)
    App.g_kKeyboardBinding.BindKey(App.WC_NUMPAD5, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_TOGGLE_SECONDARY_TARGET, App.KeyboardBinding.GET_INT_EVENT, 4, App.KeyboardBinding.KBT_SINGLE_KEY_TO_EVENT)
    App.g_kKeyboardBinding.BindKey(App.WC_NUMPAD6, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_TOGGLE_SECONDARY_TARGET, App.KeyboardBinding.GET_INT_EVENT, 5, App.KeyboardBinding.KBT_SINGLE_KEY_TO_EVENT)
    App.g_kKeyboardBinding.BindKey(App.WC_NUMPAD7, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_TOGGLE_SECONDARY_TARGET, App.KeyboardBinding.GET_INT_EVENT, 6, App.KeyboardBinding.KBT_SINGLE_KEY_TO_EVENT)
    App.g_kKeyboardBinding.BindKey(App.WC_NUMPAD8, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_TOGGLE_SECONDARY_TARGET, App.KeyboardBinding.GET_INT_EVENT, 7, App.KeyboardBinding.KBT_SINGLE_KEY_TO_EVENT)
    App.g_kKeyboardBinding.BindKey(App.WC_NUMPAD9, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_TOGGLE_SECONDARY_TARGET, App.KeyboardBinding.GET_INT_EVENT, 8, App.KeyboardBinding.KBT_SINGLE_KEY_TO_EVENT)
    App.g_kKeyboardBinding.BindKey(App.WC_NUMPAD0, App.TGKeyboardEvent.KS_KEYDOWN, ET_INPUT_TOGGLE_SECONDARY_TARGET, App.KeyboardBinding.GET_INT_EVENT, 9, App.KeyboardBinding.KBT_SINGLE_KEY_TO_EVENT)
    App.g_kEventManager.RemoveBroadcastHandler(App.ET_MISSION_START, App.Game_GetCurrentGame(), __name__ + ".MissionStart")

App.g_kEventManager.AddBroadcastPythonFuncHandler(App.ET_MISSION_START, App.Game_GetCurrentGame(), __name__ + ".MissionStart")

